﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;
using System.Windows.Forms;

namespace Trolleybus
{
    /// <summary>
    /// Класс прорисовки и перемещения объекта
    /// </summary>
    internal class DrawingTrolleybus
    {
        public EntityTrolleybus Trolleybus { get; private set; }
        /// <summary>
        /// левая координата отрисовки
        /// </summary>
        private float _startPosX;
        /// <summary>
        /// верхняя координата отрисовки
        /// </summary>
        private float _startPosY;
        /// <summary>
        /// левая координата начала отрисовки
        /// </summary>
        //private float _startX;
        /// <summary>
        /// Верхняя координата начала отрисовки
        /// </summary>
        //private float _startY;
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private int? _pictureWidth = null;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private int? _pictureHeight = null;
        /// <summary>
        /// Ширина отрисовки
        /// </summary>
        protected readonly int _trolleybusWidth = 200;
        /// <summary>
        /// Высота отрисовки
        /// </summary>
        protected readonly int _trolleybusHeight = 50;
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес автомобиля</param>
        /// <param name="bodyColor">Цвет кузова</param>
        public void Init(int speed, float weight, Color bodyColor)
        {
            Trolleybus = new EntityTrolleybus();
            Trolleybus.Init(speed, weight, bodyColor);
        }
        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
//        public void SetPosition(int x, int y, int startX, int startY, int width, int height)
        public void SetPosition(int x, int y, int width, int height)
        {
            _startPosX = x;
            _startPosY = y;
//            _startX = startX;
//            _startY = startY;
            _pictureWidth = width;
            _pictureHeight = height;
        }
        /// <summary>
        /// Изменение направвления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(Direction direction)
        {
            if (!_pictureWidth.HasValue || !_pictureHeight.HasValue)
            {
                return;
            }
            switch (direction)
            {
                //вправо
                case Direction.Right:
                    if(_startPosX + _trolleybusWidth + Trolleybus.Step < _pictureWidth)
                    {
                        _startPosX += Trolleybus.Step;
                    }
                    break;
                //влево
                case Direction.Left:
                    if (_startPosX - Trolleybus.Step > 0)
                    {
                        _startPosX -= Trolleybus.Step;
                    }
                    break;
                //вверх
                case Direction.Up:
                    if (_startPosY - Trolleybus.Step > 0)
                    {
                        _startPosY -= Trolleybus.Step;
                    }
                    break;
                //вниз
                case Direction.Down:
                    if (_startPosY + _trolleybusHeight + Trolleybus.Step < _pictureHeight)
                    {
                        _startPosY += Trolleybus.Step;
                    }
                    break;
            }
        }
        /// <summary>
        /// Отрисовка троллейбуса
        /// </summary>
        /// <param name="g"></param>
        public void DrawTransport(Graphics g)
        {
            if (_startPosX < 0 || _startPosY < 0
               || !_pictureHeight.HasValue || !_pictureWidth.HasValue)
            {
                return;
            }
            Pen pen = new Pen(Color.Black);
            Brush brBlue = new SolidBrush(Color.LightBlue);
            Brush dBlue = new SolidBrush(Color.DarkBlue);
            Brush bWhite = new SolidBrush(Color.White);
            g.DrawRectangle(pen, _startPosX, _startPosY, 200, 50);
            g.DrawRectangle(pen, _startPosX + 100, _startPosY + 10, 20, 40);
            Pen WinBlue = new Pen(Color.Blue);
            g.FillEllipse(brBlue, _startPosX, _startPosY + 5, 20, 25);
            g.DrawEllipse(WinBlue, _startPosX, _startPosY + 5, 20, 25);
            g.FillEllipse(brBlue, _startPosX + 25, _startPosY + 5, 20, 25);
            g.DrawEllipse(WinBlue, _startPosX + 25, _startPosY + 5, 20, 25);
            g.FillEllipse(brBlue, _startPosX + 50, _startPosY + 5, 20, 25);
            g.DrawEllipse(WinBlue, _startPosX + 50, _startPosY + 5, 20, 25);
            g.FillEllipse(brBlue, _startPosX + 75, _startPosY + 5, 20, 25);
            g.DrawEllipse(WinBlue, _startPosX + 75, _startPosY + 5, 20, 25);
            g.FillEllipse(brBlue, _startPosX + 120, _startPosY + 5, 20, 25);
            g.DrawEllipse(WinBlue, _startPosX + 120, _startPosY + 5, 20, 25);
            g.FillEllipse(brBlue, _startPosX + 145, _startPosY + 5, 20, 25);
            g.DrawEllipse(WinBlue, _startPosX + 145, _startPosY + 5, 20, 25);
            g.FillEllipse(brBlue, _startPosX + 170, _startPosY + 5, 20, 25);
            g.DrawEllipse(WinBlue, _startPosX + 170, _startPosY + 5, 20, 25);
            g.FillEllipse(bWhite, _startPosX, _startPosY + 40, 30, 30);
            g.DrawEllipse(pen, _startPosX, _startPosY + 40, 30, 30);
            g.FillEllipse(bWhite, _startPosX + 170, _startPosY + 40, 30, 30);
            g.DrawEllipse(pen, _startPosX + 170, _startPosY + 40, 30, 30);
        }
        /// <summary>
        /// Смена границ формы отрисовки
        /// </summary>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public void ChangeBorders(int width, int height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if(_pictureWidth <= _trolleybusWidth || _pictureHeight <= _trolleybusHeight)
            {
                _pictureWidth = null;
                _pictureHeight = null;
                return;
            }
            if (_startPosX + _trolleybusWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth.Value - _trolleybusWidth;
            }
            if (_startPosY + _trolleybusHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight.Value - _trolleybusHeight;
            }
        }
    }
}
